/* progress.c */

#include <Pilot.h>
#include <SysEvtMgr.h>

#include "pz.h"
#include "pz_rcp.h"


/*
 * static variables
 */

static const CustomPatternType gray_pat = { 0xaa55, 0xaa55, 0xaa55, 0xaa55 };


/*
 * progress functions
 */

static int progress_event_loop(void)
{
    EventType event;
    Boolean enter = false;

    EvtResetAutoOffTimer();

    while (true) {
	EvtGetEvent(&event, 0);

	if (SysHandleEvent(&event) == false) {
	    switch (event.eType) {
	    case appStopEvent:
		EvtAddEventToQueue(&event);
		return 1;

	    case ctlEnterEvent:
		enter = true;
		break;

	    case ctlSelectEvent:
		if (event.data.ctlSelect.controlID == PROGRESS_CANCEL_ID) {
		    return 1;
		}

		/* fall through */

	    case ctlExitEvent:
		enter = false;
		break;

	    case nilEvent:
		if (! enter) {
		    return 0;
		}
		break;

	    default:
		break;
	    }

	    FrmHandleEvent(FrmGetActiveForm(), &event);
	}
    }
}

int show_progress(ULong val, ULong max)
{
    RectangleType r;
    CustomPatternType saved_pattern;

    GetObjectBounds(PROGRESS_GADGET_ID, &r);

    if (max == 0) {
	WinDrawRectangleFrame(simpleFrame, &r);
    } else {
	r.extent.x = r.extent.x * val / max;
	WinGetPattern(saved_pattern);
	WinSetPattern((void *) gray_pat);
	WinFillRectangle(&r, 0);
	WinSetPattern(saved_pattern);
    }

    return (max == 0) ? 0 : progress_event_loop();
}

FormPtr start_progress(FormPtr *saved_form, CharPtr title, CharPtr name)
{
    FormPtr form;

    *saved_form = FrmGetActiveForm();

    form = FrmInitForm(PROGRESS_DIALOG_ID);
    FrmSetTitle(form, title);
    FrmCopyLabel(form, PROGRESS_LABEL2_ID, name);
    FrmSetActiveForm(form);
    FrmDrawForm(form);

    show_progress(0, 0);

    return form;
}

void end_progress(FormPtr form, FormPtr saved_form)
{
    FrmEraseForm(form);
    FrmDeleteForm(form);
    FrmSetActiveForm(saved_form);

    EvtResetAutoOffTimer();
}
