/* table.c */

#include <Pilot.h>

#include "pz.h"


/*
 * types
 */

struct table_rec {
    TablePtr table;
    ScrollBarPtr scrollbar;

    int rows;
    int items;
    int top;
    int selection;
};


/*
 * static variables
 */

static struct table_rec cur_table;


/*
 * inititlization
 */

void init_table(
	int table_id, int scrollbar_id, TableDrawItemFuncPtr draw_procedure)
{
    int i;

    cur_table.table = GetObjectPtr(table_id);
    cur_table.scrollbar = GetObjectPtr(scrollbar_id);
    cur_table.rows = TblGetNumberOfRows(cur_table.table);

    cur_table.items = 0;
    cur_table.top = 0;
    cur_table.selection = -1;

    TblSetColumnUsable(cur_table.table, 0, true);
    for (i = 0; i < cur_table.rows; ++i) {
	TblSetItemStyle(cur_table.table, i, 0, customTableItem);
    }
    TblSetCustomDrawProcedure(cur_table.table, 0, draw_procedure);
}


/*
 * drawing and scrolling functions
 */

static void restore_selection(void)
{
    if (cur_table.selection >= cur_table.top
     && cur_table.selection < cur_table.items
     && cur_table.selection < cur_table.top + cur_table.rows) {
	TblSelectItem(cur_table.table, cur_table.selection - cur_table.top, 0);
    }
}

void redraw_table(Boolean update_scrollbar)
{
    TblUnhighlightSelection(cur_table.table);
    TblDrawTable(cur_table.table);
    restore_selection();

    if (update_scrollbar) {
	SclSetScrollBar(cur_table.scrollbar, cur_table.top, 0,
		max(cur_table.items - cur_table.rows, 0), cur_table.rows);
	SclDrawScrollBar(cur_table.scrollbar);
    }
}

void scroll_table(int delta, Boolean update_scrollbar)
{
    cur_table.top += delta;

    if (cur_table.top > cur_table.items - cur_table.rows) {
	cur_table.top = cur_table.items - cur_table.rows;
    }
    if (cur_table.top < 0) {
	cur_table.top = 0;
    }

    redraw_table(update_scrollbar);
}


/*
 * accessor functions
 */

void set_table_items(int items, int top, int selection)
{
    int i;

    cur_table.items = items;
    cur_table.top = 0;
    cur_table.selection = selection;

    for (i = 0; i < cur_table.rows; ++i) {
	TblSetRowUsable(cur_table.table, i, i < cur_table.items);
    }

    if ((cur_table.selection >= 0
      && cur_table.selection < cur_table.items)
     && (cur_table.selection < top
      || cur_table.selection >= top + cur_table.rows)) {
	scroll_table(cur_table.selection - cur_table.rows / 2, true);
    } else {
	scroll_table(top, true);
    }
}

int get_table_top(void)
{
    return cur_table.top;
}

int get_table_selection(void)
{
    return cur_table.selection;
}


/*
 * event handlers
 */

Boolean handle_table_event(EventPtr event, void (*double_func)(void))
{
    int new_selection;
    Boolean handled = false;

    switch (event->eType) {
    case keyDownEvent:
	if (event->data.keyDown.chr == pageUpChr) {
	    scroll_table(- (cur_table.rows - 1), true);
	    handled = true;
	} else if (event->data.keyDown.chr == pageDownChr) {
	    scroll_table(cur_table.rows - 1, true);
	    handled = true;
	}
	break;

    case sclRepeatEvent:
	scroll_table(
		event->data.sclRepeat.newValue - event->data.sclRepeat.value,
		false);
	break;

    case tblExitEvent:
	cur_table.selection = -1;
	handled = true;
	break;

    case tblSelectEvent:
	new_selection = cur_table.top + event->data.tblSelect.row;
	if (new_selection == cur_table.selection) {
	    if (double_func != NULL) {
		(*double_func)();
	    }
	} else {
	    cur_table.selection = new_selection;
	}
	handled = true;
	break;

    default:
	break;
    }

    return handled;
}
